% Sample code of the paper:
% 
% Jiahao Pang and Gene Cheung, "Graph Laplacian regularization for image 
% denoising: analysis in the continuous domain," IEEE Trans. Image Process., 
% vol. 26, no. 4, pp. 1770-1785, April 2017.
% 
% MATLAB code prepard by Jiahao Pang
% E-mail: jpang@connect.ust.hk or pangjiahao@sensetime.com
% 
% This script shows an example of calling our denoising algorithm

close all
clear
clc

%% use parallelization to speed up
n_worker = 4; % the number of cores to use
if n_worker > 1
    myCluster=parcluster('local'); myCluster.NumWorkers = n_worker;
    matlabpool('local', n_worker) % turn on the workers
end

%% initialzation
scale = 0.5; % scaling of the original image
sigma = 20; % standard deviation of the noise
randn('seed', 0); % seed of the random number generator

%% denoising of natural image
img_ntl_ori = imresize(im2double(imread('tstimg/lena.png')), scale);
img_ntl_noi = img_ntl_ori + (sigma / 255) ...
    * randn(size(img_ntl_ori)); % add Gaussian noise
img_ntl_den = oglr(img_ntl_noi, sigma, 'ntl');
figure; subplot(1, 3, 1), imshow(img_ntl_ori), title('original');
subplot(1, 3, 2), imshow(img_ntl_noi), title('noisy');
subplot(1, 3, 3), imshow(img_ntl_den), title('denoised');
psnr_ntl_noi = 10 * log10(1 / mean((img_ntl_ori(:) - img_ntl_noi(:)).^2));
psnr_ntl_den = 10 * log10(1 / mean((img_ntl_ori(:) - img_ntl_den(:)).^2));
ssim_ntl_noi = ssim_index(im2uint8(img_ntl_ori), im2uint8(img_ntl_noi));
ssim_ntl_den = ssim_index(im2uint8(img_ntl_ori), im2uint8(img_ntl_den));
fprintf(['ntl img --- psnr_noi: %2.2f dB,  psnr_den: %2.2f dB,  ', ...
    'ssim_noi: %0.3f,  ssim_den: %0.3f\n'], ...
    psnr_ntl_noi, psnr_ntl_den, ssim_ntl_noi, ssim_ntl_den);

%% denoising of piecewise smooth image
img_pws_ori = imresize(im2double(imread('tstimg/teddy.png')), scale);
img_pws_noi = img_pws_ori + (sigma / 255) ...
    * randn(size(img_pws_ori)); % add Gaussian noise
img_pws_den = oglr(img_pws_noi, sigma, 'pws');
figure; subplot(1, 3, 1), imshow(img_pws_ori), title('original');
subplot(1, 3, 2), imshow(img_pws_noi), title('noisy');
subplot(1, 3, 3), imshow(img_pws_den), title('denoised');
psnr_pws_noi = 10 * log10(1 / mean((img_pws_ori(:) - img_pws_noi(:)).^2));
psnr_pws_den = 10 * log10(1 / mean((img_pws_ori(:) - img_pws_den(:)).^2));
ssim_pws_noi = ssim_index(im2uint8(img_pws_ori), im2uint8(img_pws_noi));
ssim_pws_den = ssim_index(im2uint8(img_pws_ori), im2uint8(img_pws_den));
fprintf(['pws img --- psnr_noi: %2.2f dB,  psnr_den: %2.2f dB,  ', ...
    'ssim_noi: %0.3f,  ssim_den: %0.3f\n'], ...
    psnr_pws_noi, psnr_pws_den, ssim_pws_noi, ssim_pws_den);

if n_worker > 1 % shut down the workers
    matlabpool close
end
